// TrackDemoSheet.cpp : implementation file
//

#include "stdafx.h"
#include "TrackDemo.h"
#include "TrackDemoSheet.h"
#include "..\VSMSGW\VSMSGW.h"
#include "Setup.h"
#include "DlgGPRSSetup.h"
#include "DlgGatewaySetup.h"

static UINT BASED_CODE indicators[] =
{
  ID_INDICATOR_STATUS,
  ID_INDICATOR_NODEID,
  ID_INDICATOR_TIME
};

//
// Utility functions to handle conversion from Unicode Windows application
// to ANSI dll libraries and RTCU units.
//

void TextToDLL(const CString &text, char *dlltext, int maxsize)
{
#ifdef _UNICODE
  //First call the function to determine how much space we need to allocate
  int nLength = text.GetLength();
  int dllLength = WideCharToMultiByte(CP_UTF8, 0, text, nLength, NULL, 0, NULL, NULL);
  if (nLength >= maxsize) nLength = maxsize - 1;

  //Now recall with the buffer to get the converted text
	int nCharsWritten = WideCharToMultiByte(CP_UTF8, 0, text, nLength, dlltext, dllLength, NULL, NULL);
  dlltext[nCharsWritten] = '\0';
#else
  strcpy_s(dlltext, maxsize, text);
#endif
}

CString TextFromDLL(const char *dlltext, int nLength)
{
#ifdef _UNICODE
  //First call the function to determine how much space we need to allocate
	int nWideLength = MultiByteToWideChar(CP_UTF8, 0, dlltext, nLength, NULL, 0);

  //If the calculated length is zero, then ensure we have at least room for a NULL terminator
  if (nWideLength == 0) nWideLength = 1;

  //Now recall with the buffer to get the converted text
  CString sWideString;
  wchar_t* pszWText = sWideString.GetBuffer(nWideLength + 1); //include an extra byte because we may be null terminating the string ourselves
	int nCharsWritten = MultiByteToWideChar(CP_UTF8, 0, dlltext, nLength, pszWText, nWideLength);
	
	//Ensure we NULL terminate the text if MultiByteToWideChar doesn't do it for us
	if (nLength != -1)
	{
	  AFXASSUME(nCharsWritten <= nWideLength);
	  pszWText[nCharsWritten] = '\0';
	}
	
	sWideString.ReleaseBuffer();

	return sWideString;
#else
  return CString(dlltext);
#endif
}

//
// Function that is called when receiving data from the RTCU unit
//

static int _cdecl cbPDU(int HisNodeID, unsigned char *data, int length, void *arg)
{
  // Look at the command code:
  switch (data[0]){
    case 0x09:
      // Event information
      ((CTrackDemoSheet*)arg)->ReceiveEvent(CTime::GetCurrentTime(), HisNodeID, &data[1]);
      break;

    case 0x10:
      // Roaming list
      ((CTrackDemoSheet*)arg)->ReceiveRoaming((unsigned int*)&data[1]);
      break;

    case 0x07:
      // Powersave
      ((CTrackDemoSheet*)arg)->ReceivePowersave(data);
      break;
  }

  return 0;
}

// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
  CAboutDlg();

// Dialog Data
  enum { IDD = IDD_ABOUTBOX };

protected:
  virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

// Implementation
protected:
  DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()


// CTrackDemoSheet

IMPLEMENT_DYNAMIC(CTrackDemoSheet, CPropertySheet)

CTrackDemoSheet::CTrackDemoSheet(void)
  :CPropertySheet(_T("Advanced Tracking Demo"), NULL, 0)
  , m_connected(true)
  , m_client(0)
{
  // Insert pages
  AddPage(&m_events);
  AddPage(&m_power);
  AddPage(&m_roaming);
  AddPage(&m_actions);
}

CTrackDemoSheet::~CTrackDemoSheet()
{
}

void CTrackDemoSheet::DoDataExchange(CDataExchange* pDX)
{
  CPropertySheet::DoDataExchange(pDX);
  DDX_Text(pDX, IDC_NODEID, m_client);
}


BEGIN_MESSAGE_MAP(CTrackDemoSheet, CPropertySheet)
  ON_COMMAND(IDM_ABOUTBOX, &CTrackDemoSheet::OnAboutbox)
  ON_COMMAND(ID_SETUP, &CTrackDemoSheet::OnSetup)
  ON_WM_TIMER()
  ON_COMMAND(ID_UNIT_PROVIDERGPRSACCESS, &CTrackDemoSheet::OnUnitProvidergprsaccess)
  ON_COMMAND(ID_UNIT_RTCUGATEWAYACCESS, &CTrackDemoSheet::OnUnitRtcugatewayaccess)
  ON_COMMAND(ID_UNIT_SETHOSTNODEID, &CTrackDemoSheet::OnUnitSethostnodeid)
  ON_COMMAND(ID_UNIT_REQUESTPOSITION, &CTrackDemoSheet::OnUnitRequestposition)
  ON_EN_CHANGE(IDC_NODEID, &CTrackDemoSheet::OnChangeNodeid)
  ON_BN_CLICKED(IDC_EXPORT, &CTrackDemoSheet::OnExport)
END_MESSAGE_MAP()


// CTrackDemoSheet message handlers

BOOL CTrackDemoSheet::OnInitDialog() 
{
  // Var
  CRect rect;
  int   rc, top, edge;
  char  gw_addr[127], gw_key[9];
  unsigned char CryptKey[16] = {0};
  CString text;

  // Default
  BOOL bResult = CPropertySheet::OnInitDialog();

  // Hide un-used buttons
  GetDlgItem(IDOK)->ShowWindow(false);
  GetDlgItem(IDHELP)->ShowWindow(false);
  GetDlgItem(ID_APPLY_NOW)->ShowWindow(false);
  GetDlgItem(IDCANCEL)->ShowWindow(false);

  // Resize the dialog
  GetWindowRect(&rect);
  rect.bottom += 42;
  MoveWindow(&rect);

  // Calculate control positions
  GetClientRect(&rect);
  top = rect.bottom - 72;
  GetTabControl()->GetWindowRect(&rect);
  ScreenToClient(&rect);
  edge = rect.right - 80;

  // Initialize controls
  text.Format(_T("%d"), AfxGetApp()->GetProfileInt(_T("Options"), _T("His nodeid"), 2001));
  c_label.CreateEx(0, _T("STATIC"), _T("Client Node ID:"), WS_CHILD|WS_VISIBLE|SS_CENTERIMAGE, 10, top, 75, 23, m_hWnd, 0, 0);
  c_nodeid.CreateEx(WS_EX_CLIENTEDGE, _T("EDIT"), text, WS_CHILD|WS_VISIBLE|WS_TABSTOP|ES_NUMBER, 90, top, 80, 23, m_hWnd, (HMENU)IDC_NODEID, 0);
  c_export.CreateEx(0, _T("BUTTON"), _T("Export"), WS_CHILD|WS_VISIBLE|WS_TABSTOP, edge, top, 80, 23, m_hWnd, (HMENU)IDC_EXPORT, 0);

	c_label.SetFont(GetFont());
	c_nodeid.SetFont(GetFont());
	c_export.SetFont(GetFont());
  c_nodeid.LimitText(9);

  // Initialize menu
  m_menu.LoadMenu(IDR_TRACKDEMO);
  SetMenu(&m_menu);

  // Initialize status bar
  m_bar.Create(this);                   //We create the status bar
  m_bar.SetIndicators(indicators, 3);   //Set the number of panes 
  GetClientRect(&rect);                 //Size the three panes
  m_bar.SetPaneInfo(0, ID_INDICATOR_STATUS, SBPS_NORMAL, rect.Width() - 130);
  m_bar.SetPaneInfo(1, ID_INDICATOR_NODEID, SBPS_NORMAL, 50);      
  m_bar.SetPaneInfo(1, ID_INDICATOR_TIME,   SBPS_STRETCH, 0);
  RepositionBars(AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST, ID_INDICATOR_TIME);   //This is where we actually draw it on the screen

  m_bar.SetPaneText(1, _T(""));
  m_bar.SetPaneText(2, _T(""));

  // Initialize Gateway connection
  TextToDLL(AfxGetApp()->GetProfileString(_T("Options"), _T("GW IP"), _T("gw.rtcu.dk")), gw_addr, 127);
  TextToDLL(AfxGetApp()->GetProfileString(_T("Options"), _T("GW Key"), _T("AABBCCDD")), gw_key, 9);
  rc = vsmsgwInitAdv(
                     AfxGetApp()->GetProfileInt(_T("Options"), _T("My nodeid"), 2000),
                     gw_addr,
                     AfxGetApp()->GetProfileInt(_T("Options"), _T("GW Port"), 5001),
                     gw_key,
                     CryptKey
                    );
  vsmsgwSetSMSPDUCB(cbPDU, this);
  SetTimer(100, 500, NULL);   // Connection status update timer

  // Completed
  return bResult;
}

void CTrackDemoSheet::OnTimer(UINT_PTR nIDEvent)
{
  // Var
  bool connected;

  // Check connection status
  if (nIDEvent == 100) {
    connected = vsmsgwIsConnected();
    if (connected != m_connected) {
      // Connection status is changed
      unsigned long MyNodeID;
      CString s;
      // Write status text
      m_bar.SetPaneText(0, connected ? _T("Connected to RTCU Gateway") : _T("Connecting to RTCU Gateway...."));
      // Write Node ID
      vsmsgwGetMyNodeID(&MyNodeID);
      s.Format(_T("%d"), MyNodeID);
      m_bar.SetPaneText(1, connected ? s : _T(""));
      // Set timestamp
      if (connected) m_time = CTime::GetCurrentTime();
      else m_bar.SetPaneText(2, _T(""));
    }
    m_connected = connected;

    // Update time
    if (connected) {
      CTimeSpan t1 = CTime::GetCurrentTime() - m_time;
      m_bar.SetPaneText(2, t1.Format(_T("%H:%M:%S")));
    }
  }

  CPropertySheet::OnTimer(nIDEvent);
}

void CTrackDemoSheet::OnAboutbox()
{
  CAboutDlg dlgAbout;
  dlgAbout.DoModal();
}

void CTrackDemoSheet::OnSetup()
{
  // Var
  CSetup dlg;

  // Fill dialog
  dlg.m_addr    = AfxGetApp()->GetProfileString(_T("Options"), _T("GW IP"), _T("gw.rtcu.dk"));
  dlg.m_port    = AfxGetApp()->GetProfileInt(_T("Options"), _T("GW Port"), 5001);
  dlg.m_key     = AfxGetApp()->GetProfileString(_T("Options"), _T("GW Key"), _T("AABBCCDD"));
  dlg.m_nodeid  = AfxGetApp()->GetProfileInt(_T("Options"), _T("My nodeid"), 2000);

  // Show dialog
  if (dlg.DoModal() == IDOK) {
    // Store new setup
    AfxGetApp()->WriteProfileString(_T("Options"), _T("GW IP"), dlg.m_addr);
    AfxGetApp()->WriteProfileInt(_T("Options"), _T("GW Port"), dlg.m_port);
    AfxGetApp()->WriteProfileString(_T("Options"), _T("GW Key"), dlg.m_key);
    AfxGetApp()->WriteProfileInt(_T("Options"), _T("My nodeid"), dlg.m_nodeid);
    // Show restart message
    MessageBox(_T("The program must be restarted in order to use any new settings !"), _T("Advanced Trancking Demo"), MB_OK|MB_ICONASTERISK);
  }
}

void CTrackDemoSheet::OnUnitProvidergprsaccess()
{
  // Show dialog
  CDlgGPRSSetup dlg;
  if (dlg.DoModal() == IDOK) {
    // Var
    union Frames req;
    int          rc;

    // Create command VSMS
    req.gprs.Command  = 0x12;
    req.gprs.Index    = dlg.m_Index + 1;
    req.gprs.Lai      = dlg.m_Entry[dlg.m_Index].LAI;
    req.gprs.Ip       = dlg.m_Entry[dlg.m_Index].IP;
    req.gprs.Subnet   = dlg.m_Entry[dlg.m_Index].Subnet;
    req.gprs.Gateway  = dlg.m_Entry[dlg.m_Index].Gateway;
    req.gprs.DNS1     = dlg.m_Entry[dlg.m_Index].DNS1;
    req.gprs.DNS2     = dlg.m_Entry[dlg.m_Index].DNS2;
    rc                = 26;
    req.data[rc++]    = dlg.m_Entry[dlg.m_Index].Username.GetLength();
    TextToDLL(dlg.m_Entry[dlg.m_Index].Username, &req.data[rc], 140 - rc);
    rc               += req.data[rc - 1];
    req.data[rc++]    = dlg.m_Entry[dlg.m_Index].Password.GetLength();
    TextToDLL(dlg.m_Entry[dlg.m_Index].Password, &req.data[rc], 140 - rc);
    rc               += req.data[rc - 1];
    req.data[rc++]    = dlg.m_Entry[dlg.m_Index].APN.GetLength();
    TextToDLL(dlg.m_Entry[dlg.m_Index].APN, &req.data[rc], 140 - rc);

    // Send VSMS
    if (vsmsgwSendPDU(AfxGetApp()->GetProfileInt(_T("Options"), _T("His nodeid"), 2001), (unsigned char*)&req.data, sizeof(req.data), &rc) || rc)
      MessageBox(_T("No responce from unit"), _T("GPRS access parameters"), MB_OK|MB_ICONWARNING);
  }
}

void CTrackDemoSheet::OnUnitRtcugatewayaccess()
{
  // Show dialog
  CDlgGatewaySetup dlg;
  if (dlg.DoModal() == IDOK) {
    // Var
    union Frames req;
    int          rc;

    // Create command VSMS
    req.gw.Command    = 0x14;
    req.gw.Enable     = dlg.m_enable;
    req.gw.Port       = dlg.m_port;
    req.gw.MaxConn    = dlg.m_maxconn;
    req.gw.Retries    = dlg.m_retries;
    req.gw.Timeout    = dlg.m_timeout;
    req.gw.Alive      = dlg.m_alive;
    rc                = 16;
    req.data[rc++]    = dlg.m_ip.GetLength();
    TextToDLL(dlg.m_ip, &req.data[rc], 140 - rc);
    rc               += req.data[rc - 1];
    req.data[rc++]    = dlg.m_key.GetLength();
    TextToDLL(dlg.m_key, &req.data[rc], 140 - rc);

    // Send VSMS
    if (vsmsgwSendPDU(AfxGetApp()->GetProfileInt(_T("Options"), _T("His nodeid"), 2001), (unsigned char*)&req.data, sizeof(req.data), &rc) || rc)
      MessageBox(_T("No responce from unit"), _T("RTCU Gateway parameters"), MB_OK|MB_ICONWARNING);
  }
}

void CTrackDemoSheet::OnUnitSethostnodeid()
{
  // Var
  struct FrameHostID req;
  int rc;

  // Build frame
  req.Command=0x13;
  vsmsgwGetMyNodeID(&req.NodeID);

  if (vsmsgwSendPDU(AfxGetApp()->GetProfileInt(_T("Options"), _T("His nodeid"), 2001), (unsigned char*)&req, sizeof(req), &rc) || rc)
    MessageBox(_T("No responce from unit"), _T("Set Host NodeId"), MB_OK|MB_ICONWARNING);
}

void CTrackDemoSheet::OnUnitRequestposition()
{
  // Var
  unsigned char req = 0x02;
  int rc;

  // Send request
  if (vsmsgwSendPDU(AfxGetApp()->GetProfileInt(_T("Options"), _T("His nodeid"), 2001), &req, 1, &rc) || rc)
    MessageBox(_T("No responce from unit"), _T("Request GPS position"), MB_OK|MB_ICONWARNING);
}

void CTrackDemoSheet::ReceiveEvent(CTime timestamp, int NodeID, void *eventdata)
{
  struct FrameEventInfo *pData = new struct FrameEventInfo;
  memcpy(pData, eventdata, sizeof(struct FrameEventInfo));
  m_events.AddEvent(timestamp, NodeID, pData);
}

void CTrackDemoSheet::ReceiveRoaming(unsigned int *roamingdata)
{
  m_roaming.OnReceive(roamingdata);
}

void CTrackDemoSheet::ReceivePowersave(void *pData)
{
  m_power.OnReceive((FrameSetPowersave*)pData);
}

void CTrackDemoSheet::OnChangeNodeid()
{
  UpdateData(TRUE);
  AfxGetApp()->WriteProfileInt(_T("Options"), _T("His nodeid"), m_client);
}

void CTrackDemoSheet::OnExport()
{
  // Var
  CFileDialog dlg(FALSE, _T("txt"), NULL, OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT, _T("Comma separated file|*.txt|KML route file|*.kml||"), this);

  // Show dialog
  if (dlg.DoModal() == IDOK) {
    if (!dlg.GetFileExt().CompareNoCase(_T("kml")))
      m_events.ExportKml(dlg.GetPathName());
    else
      m_events.Export(dlg.GetPathName());
  }
}
